#----------------------------------------------------------------------
#  dungogcron.pm
#  support@dungog.net
#----------------------------------------------------------------------

package esmith::FormMagick::Panel::dungogcron;

use strict;
use warnings;

use esmith::FormMagick;
use esmith::AccountsDB;

use Exporter;
use Carp qw(verbose);

use HTML::Tabulate;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw();

our $adb = esmith::AccountsDB->open();
our $ddb = esmith::ConfigDB->open('dungog') or die "Could not open dungog DB\n";

sub new {
    shift;
    my $self = esmith::FormMagick->new();
    $self->{calling_package} = (caller)[0];
    bless $self;
    return $self;
}

sub get_user
{
    my @users = $adb->get_all_by_prop(type => "user");
    my @unames = ('root');
    foreach my $u (sort @users) 
    {
       my $name = $u->key();
       push (@unames, $name)
    }

    return [@unames];
    #return ['root', 'stephen'];
}

sub print_cron_table
{
    my $self = shift;
    my $q = $self->{cgi};

    my %Labelday = ("monthly" => $self->localise("9MONTHLY"),
                        "*"   => $self->localise("DAILY"),
                        "0-6" => $self->localise("DAILY"),
                        "1-5" => $self->localise("WEEKDAYS"),
                        "sun" => $self->localise("0SUNDAY"),
                        "1"   => $self->localise("1MONDAY"),
                        "2"   => $self->localise("2TUESDAY"),
                        "3"   => $self->localise("3WEDNESDAY"),
                        "4"   => $self->localise("4THURSDAY"),
                        "5"   => $self->localise("5FRIDAY"),
                        "6"   => $self->localise("6SATURDAY"));

    my $cron_table = 
    {
       title => $self->localise("CURRENT_LIST"),

       stripe => "#D4D0C8",

       fields => [ qw(desc command freq dday time Modify Run Remove) ],

       labels => 1,

       field_attr => {
                       desc          => {    label => $self->localise("DESCRIPTION")    },

                       command       => {    label => $self->localise("COMMAND")        },

                       freq          => {    label => $self->localise("ENABLED")        },

                       dday          => {    label => $self->localise("DAY")            },

                       time          => {    label => $self->localise("TIME")           },

                       Modify        => {  
                                          label => $self->localise("MODIFY"),
                                          link  => \&modify_link
                                        },

                       Run           => {  
                                          label => $self->localise("RUN"),
                                          link  => \&command_link
                                        },

                       Remove        => {  
                                          label => $self->localise("REMOVE"),
                                          link  => \&remove_link
                                        },
                      }
    };

    my @data = ();

    foreach my $cron ($ddb->get_all_by_prop(type => "Cron"))
    {
        my $key   = $cron->key;
        my $day   = $cron->prop("day") || '';
        my $hour  = $cron->prop("hour")|| '0';
        my $recur = $cron->prop("recur") || '';
        my $min   = $cron->prop("min") || '0';
        my $freq  = $cron->prop("freq") || '';
        my $oride = $cron->prop("oride") || '';
        my $time  = '';
        my $dday  = '';

        if ($freq eq 'on')
        {
           $dday = ($Labelday{$day});

           if ($oride eq 'yes')
           { $time = 'override'; }
           else
           { $time = $hour.$recur.':'.$min  }
        }
        else
        { $freq = '';}

        $day = '0-6' if ($day eq '*');

        push @data,
            {
              desc     => $cron->prop("desc") || '',
              freq     => $freq,
              day      => $day,
              dday     => $dday,
              hour     => $hour,
              recur    => $recur,
              min      => $min,
              oride    => $oride,
              command  => $cron->prop("command") || '',
              mail     => $cron->prop("mail") || '',
              user     => $cron->prop("user") || 'root',
              time     => $time,
              key      => $key,
              Modify   => $self->localise('MODIFY'),
              Run      => $self->localise('RUN'),
              Remove   => $self->localise('REMOVE'),
            }
    }

    my $t = HTML::Tabulate->new($cron_table);

    $t->render(\@data, $cron_table);
}

sub modify_link
{
    my ($data_item, $row, $field) = @_;

    return "dungog-cron?" .
        join("&",
        "page=0",
        "page_stack=",
        "Next=Next",
        "key="     . $row->{key},
        "desc="    . $row->{desc},
        "command=" . $row->{command},
        "freq="    . $row->{freq},
        "min="     . $row->{min},
        "hour="    . $row->{hour},
        "recur="   . $row->{recur},
        "day="     . $row->{day},
        "user="    . $row->{user},
        "mail="    . $row->{mail},
        "oride="   . $row->{oride},
        "wherenext=CRON_MODIFY");
}

sub get_minute
{
    return [0 .. 59, '*/5', '*/10', '*/15', '*/30'];
}

sub remove_link
{
    my ($data_item, $row, $field) = @_;

    return "dungog-cron?" .
        join("&",
        "page=0",
        "page_stack=",
        "Next=Next",
        "key="    . $row->{key},
        "desc="   . $row->{desc},
        "wherenext=CRON_REMOVE");
}

sub remove_cron
{
    my $self = shift;
    my $q = $self->{cgi};

    my $cron = $q->param('key');

    my $rec = $ddb->get($cron);
    $rec->delete;

    unless ( system ("/sbin/e-smith/expand-template /etc/crontab") == 0 )
    {
        $self->error('ERROR_UPDATING');
        return undef;
    }

    return $self->success("SUCCESS");
}

sub print_etc_crontab
{
    my $file = "/etc/crontab"; 
    my $text = '';

    # if exists and is not empty
    if (( -e $file ) && (! -z $file ))
    {
        open (INF, "<$file")
          or warn "Error: Could not open file: $file\n";
        my @Temp;

        while (<INF>)
        {
          $_ =~ s/</&lt\;/;
          $_ =~ s/>/&gt\;/;
          push (@Temp, $_);
        }

        $text = join ("<br>", @Temp); 

        close INF;
    }

    return $text;
}

sub change_settings
{
    my $self = shift;
    my $q = $self->{cgi};

    my $cron = $q->param('key') ||'';
    if (($cron eq 'new') || ($cron eq ''))
    {
      my $random = int(rand(999999));
      $cron = 'Cron'.$random;
      $ddb->new_record($cron, { type => "Cron" });
    }

    foreach ('desc','command','freq','min','recur','hour','day','user','mail','oride')
    { $ddb->set_prop($cron, "$_", $q->param("$_")); }

    unless ( system ("/sbin/e-smith/expand-template /etc/crontab") == 0 )
    {
        $self->error('ERROR_UPDATING');
        return undef;
    }

    return $self->success("SUCCESS");
}

sub nonblank2
{
    my ($fm, $data) = @_;
    if (not $data)
    {
        return "FM_NONBLANK"; 
    }
    elsif
    (
        $data =~ /^\s+$/ ) {
        return "FM_NONBLANK"; 
    }
    else
    {
        return "OK";
    }
}

sub command_link
{
    my ($data_item, $row, $field) = @_;

    return "dungog-cron?" .
        join("&",
        "page=0",
        "page_stack=",
        "Next=Next",
        "desc="    . $row->{desc},
        "command=" . $row->{command},
        "wherenext=PRINT_COMMAND");
}

sub print_command
{
	my $self = shift;
	my $q = $self->{cgi};

	my $desc     = $q->param ('desc')    || 'Description';
	my $command  = $q->param ('command') || 'ls /';

        if ($command =~ /^([\"\[\]\/\ \-\:\_\.A-Za-z0-9]*)$/)
	{
	    $command = $1;
	}
	else
	{
            print $q->h3($self->localise("$desc"));
            print "Disallowed Characters in Command\n";
            return $self->error('INVALID_COMMAND'); #don't think this works
	}

	print $q->h3($self->localise("$desc"));

	print "$command\n";

	open(REPORT, "$command |");

	print "<pre>\n";

	while (<REPORT>)
	{
		s/\&/\&amp;/g;
		s/\"/\&quot;/g;
		s/\>/\&gt;/g;
		s/\</\&lt;/g;

		print;
	}

	print "</pre>\n";

	close REPORT;

	print $q->h3($self->localise('END_OF_REPORT'));
	return '';
}

1;
